@@ -1,12 +1,15 @@
 package main
 
 import (
+	"context"
 	"crypto/tls"
+
 	"github.com/caddyserver/certmagic"
 )
 
 func acmeTLSConfig(domains []string, email string, disableHTTP bool, disableTLSALPN bool,
-	altHTTPPort int, altTLSALPNPort int) (*tls.Config, error) {
+	altHTTPPort int, altTLSALPNPort int,
+) (*tls.Config, error) {
 	certmagic.DefaultACME.Agreed = true
 	certmagic.DefaultACME.Email = email
 	certmagic.DefaultACME.DisableHTTPChallenge = disableHTTP
@@ -14,5 +17,5 @@ func acmeTLSConfig(domains []string, email string, disableHTTP bool, disableTLSA
 	certmagic.DefaultACME.AltHTTPPort = altHTTPPort
 	certmagic.DefaultACME.AltTLSALPNPort = altTLSALPNPort
 	cfg := certmagic.NewDefault()
-	return cfg.TLSConfig(), cfg.ManageSync(domains)
+	return cfg.TLSConfig(), cfg.ManageSync(context.Background(), domains)
 }
